"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var FCS_1 = require("./FCS");
var fcs;
beforeEach(function () {
    fcs = new FCS_1.FCS();
});
describe('Test flags', function () {
    describe('FLAG_INACTIVE', function () {
        test('Always return false', function () {
            fcs.setFlags({
                'my-feature': { flag: FCS_1.FCS.FLAG_INACTIVE },
            });
            expect(fcs.feature('my-feature')).toBe(false);
        });
    });
    describe('FLAG_ACTIVE', function () {
        test('Always return true', function () {
            fcs.setFlags({
                'my-feature': { flag: FCS_1.FCS.FLAG_ACTIVE },
            });
            expect(fcs.feature('my-feature')).toBe(true);
        });
    });
    describe('FLAG_COOKIE', function () {
        test('Return false if cookie is not defined', function () {
            fcs.setFlags({
                'my-feature': { flag: FCS_1.FCS.FLAG_COOKIE },
            });
            fcs.setCookies({});
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return true if cookie is defined', function () {
            fcs.setFlags({
                'my-feature': { flag: FCS_1.FCS.FLAG_COOKIE },
            });
            fcs.setCookies({
                'fcs-my-feature': 1,
            });
            expect(fcs.feature('my-feature')).toBe(true);
        });
        test('Accept only cookies with double dash instead of slash', function () {
            fcs.setFlags({
                'test/my-feature': { flag: FCS_1.FCS.FLAG_COOKIE },
                'test/my-feature2': { flag: FCS_1.FCS.FLAG_COOKIE },
            });
            fcs.setCookies({
                'fcs-test/my-feature': 1,
                'fcs-test--my-feature2': 1,
            });
            expect(fcs.feature('test/my-feature')).toBe(false);
            expect(fcs.feature('test/my-feature2')).toBe(true);
        });
    });
    describe('FLAG_TAG', function () {
        test('Return false if tag is not defined', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_TAG,
                    name: 'country',
                    value: 'UK',
                },
            });
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return false if tag is defined with different value', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_TAG,
                    name: 'country',
                    value: 'UK',
                },
            });
            fcs.setTag('country', 'DE');
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return true if tag is defined with the same value', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_TAG,
                    name: 'country',
                    value: 'UK',
                },
            });
            fcs.setTag('country', 'UK');
            expect(fcs.feature('my-feature')).toBe(true);
        });
        test('Return false if tag is not defined in array of options', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_TAG,
                    name: 'country',
                    value: ['UK', 'DE'],
                },
            });
            fcs.setTag('country', 'FR');
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return true if tag is defined in array of options', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_TAG,
                    name: 'country',
                    value: ['UK', 'DE'],
                },
            });
            fcs.setTag('country', 'UK');
            expect(fcs.feature('my-feature')).toBe(true);
        });
    });
    describe('FLAG_MULTI_AND', function () {
        test('Return false if definition of sub-flags is missing', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_AND,
                },
            });
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return true if both sub-conditions are met', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_AND,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.com',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'en' },
                    ],
                },
            });
            fcs.setTag('domain', 'domain.com');
            fcs.setTag('lang', 'en');
            expect(fcs.feature('my-feature')).toBe(true);
        });
        test('Return false if any of sub-conditions is not met', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_AND,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.de',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'de' },
                    ],
                },
                'my-feature2': {
                    flag: FCS_1.FCS.FLAG_MULTI_AND,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.com',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'en' },
                    ],
                },
                'my-feature3': {
                    flag: FCS_1.FCS.FLAG_MULTI_AND,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.fr',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'fr' },
                    ],
                },
            });
            fcs.setTag('domain', 'domain.de');
            fcs.setTag('lang', 'en');
            expect(fcs.feature('my-feature')).toBe(false);
            expect(fcs.feature('my-feature2')).toBe(false);
            expect(fcs.feature('my-feature3')).toBe(false);
        });
    });
    describe('FLAG_MULTI_OR', function () {
        test('Return false if definition of sub-flags is missing', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_OR,
                },
            });
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return true if ay of sub-conditions is met', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_OR,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.de',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'de' },
                    ],
                },
                'my-feature2': {
                    flag: FCS_1.FCS.FLAG_MULTI_OR,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.de',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'en' },
                    ],
                },
                'my-feature3': {
                    flag: FCS_1.FCS.FLAG_MULTI_OR,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.com',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'de' },
                    ],
                },
            });
            fcs.setTag('domain', 'domain.de');
            fcs.setTag('lang', 'de');
            expect(fcs.feature('my-feature')).toBe(true);
            expect(fcs.feature('my-feature2')).toBe(true);
            expect(fcs.feature('my-feature3')).toBe(true);
        });
        test('Return false if none of sub-conditions is met', function () {
            fcs.setFlags({
                'my-feature': {
                    flag: FCS_1.FCS.FLAG_MULTI_OR,
                    flags: [
                        {
                            flag: FCS_1.FCS.FLAG_TAG,
                            name: 'domain',
                            value: 'domain.com',
                        },
                        { flag: FCS_1.FCS.FLAG_TAG, name: 'lang', value: 'en' },
                    ],
                },
            });
            fcs.setTag('domain', 'domain.de');
            fcs.setTag('lang', 'de');
            expect(fcs.feature('my-feature')).toBe(false);
        });
    });
    describe('Missing/invalid', function () {
        test('Return false if definition of flags is not defined', function () {
            fcs.setFlags({});
            expect(fcs.feature('my-feature')).toBe(false);
        });
        test('Return false if definition of flags is invalid', function () {
            fcs.setFlags({
                // @ts-ignore
                'my-feature': {},
                // @ts-ignore
                'my-feature2': { flag: 'test.js.ts.js' },
            });
            expect(fcs.feature('my-feature')).toBe(false);
            expect(fcs.feature('my-feature2')).toBe(false);
        });
    });
});
describe('Cache', function () {
    var spy;
    beforeEach(function () {
        spy = jest.spyOn(fcs, 'isFlagActive');
    });
    afterEach(function () {
        spy.mockRestore();
    });
    test('Ensure that result of feature assessment is not cached when tag is updated', function () {
        fcs.setFlags({
            'my-feature': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'country',
                value: 'UK',
            },
        });
        fcs.setTag('country', 'UK');
        expect(fcs.feature('my-feature')).toBe(true);
        fcs.setTag('country', 'DE');
        expect(fcs.feature('my-feature')).toBe(false);
    });
    test('Ensure that result of feature assessment is not cached anymore when clearCache is used', function () {
        fcs.setFlags({
            'my-feature': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'country',
                value: 'UK',
            },
        });
        fcs.setTag('country', 'UK');
        fcs.feature('my-feature');
        expect(spy).toHaveBeenCalledTimes(1);
        fcs.feature('my-feature');
        expect(spy).toHaveBeenCalledTimes(1);
        fcs.clearCache();
        fcs.feature('my-feature');
        expect(spy).toHaveBeenCalledTimes(2);
    });
});
describe('getActiveFeatures', function () {
    test('Expect to return only features that meet conditions of flags', function () {
        fcs.setFlags({
            'my-feature': { flag: FCS_1.FCS.FLAG_ACTIVE },
            'my-feature2': { flag: FCS_1.FCS.FLAG_INACTIVE },
            'my-feature3': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'domain',
                value: 'domain.com',
            },
            'my-feature4': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'lang',
                value: 'fr',
            },
        });
        fcs.setTag('domain', 'domain.com');
        fcs.setTag('lang', 'en');
        var activeFeatures = fcs.getActiveFeatures();
        expect(activeFeatures).toEqual(['my-feature', 'my-feature3']);
    });
});
describe('getBodyCssClasses', function () {
    test('Expect to return only CSS classes of features that meet conditions of flags', function () {
        fcs.setFlags({
            'my-feature': { flag: FCS_1.FCS.FLAG_ACTIVE },
            'my-feature2': { flag: FCS_1.FCS.FLAG_INACTIVE },
            'my-feature3': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'domain',
                value: 'domain.com',
            },
            'my-feature4': {
                flag: FCS_1.FCS.FLAG_TAG,
                name: 'lang',
                value: 'fr',
            },
        });
        fcs.setTag('domain', 'domain.com');
        fcs.setTag('lang', 'en');
        var cssClasses = fcs.getBodyCssClasses();
        expect(cssClasses).toEqual('fcs-my-feature fcs-my-feature3');
    });
    test('Expect to automatically turn slashes into double dashes in names of returned CSS classes', function () {
        fcs.setFlags({
            'test/my-feature': { flag: FCS_1.FCS.FLAG_ACTIVE },
        });
        var cssClasses = fcs.getBodyCssClasses();
        expect(cssClasses).toEqual('fcs-test--my-feature');
    });
});
describe('turnSlashesIntoDoubleDashes', function () {
    test('Expect to turn all the slashes in the provided string into double dashes', function () {
        expect(fcs.turnSlashesIntoDoubleDashes('a/b/c/d')).toEqual('a--b--c--d');
    });
});
